using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

namespace store_starter
{
	/// <summary>
	/// The codebehind for the check out page which handles all check out from billing/shipping addresses to confirmation
	/// </summary>
	public class CheckOut : System.Web.UI.Page, IStorePage {

		private StoreContext storeContext;

		#region Web Forms Designer generated code
		protected System.Web.UI.WebControls.PlaceHolder billingAddress;
		protected System.Web.UI.WebControls.PlaceHolder shippingAddress;
		protected System.Web.UI.WebControls.Repeater addressRepeater;
		protected System.Web.UI.WebControls.PlaceHolder address;
		protected System.Web.UI.WebControls.Repeater creditCardRepeater;
		protected System.Web.UI.WebControls.PlaceHolder creditCard;
		protected System.Web.UI.WebControls.PlaceHolder shippingOptions;
		protected System.Web.UI.WebControls.Button confirmButton;
		protected System.Web.UI.WebControls.TextBox street;
		protected System.Web.UI.WebControls.RequiredFieldValidator street_val2;
		protected System.Web.UI.WebControls.TextBox city;
		protected System.Web.UI.WebControls.RequiredFieldValidator city_val2;
		protected System.Web.UI.WebControls.DropDownList state;
		protected System.Web.UI.WebControls.TextBox zip;
		protected System.Web.UI.WebControls.TextBox zip4;
		protected System.Web.UI.WebControls.RequiredFieldValidator zip_val2;
		protected System.Web.UI.WebControls.RegularExpressionValidator zip_val3;
		protected System.Web.UI.WebControls.RegularExpressionValidator zip_val4;
		protected System.Web.UI.WebControls.TextBox phone;
		protected System.Web.UI.WebControls.RequiredFieldValidator phone_val2;
		protected System.Web.UI.WebControls.RegularExpressionValidator phone_val;
		protected System.Web.UI.WebControls.Button newAddressButton;
		protected System.Web.UI.WebControls.TextBox name;
		protected System.Web.UI.WebControls.RequiredFieldValidator fname_val2;
		protected System.Web.UI.WebControls.TextBox email;
		protected System.Web.UI.WebControls.RequiredFieldValidator email_val2;
		protected System.Web.UI.WebControls.TextBox cardName;
		protected System.Web.UI.WebControls.RequiredFieldValidator cardName_val;
		protected System.Web.UI.WebControls.DropDownList cardType;
		protected System.Web.UI.WebControls.RequiredFieldValidator cardType_val;
		protected System.Web.UI.WebControls.DropDownList expireMonth;
		protected System.Web.UI.WebControls.DropDownList expireYear;
		protected System.Web.UI.WebControls.TextBox cardNumber;
		protected System.Web.UI.WebControls.RequiredFieldValidator cardNumber_val;
		protected System.Web.UI.WebControls.RegularExpressionValidator cardNumber_val2;
		protected System.Web.UI.WebControls.Button newCardButton;
		protected System.Web.UI.WebControls.Repeater packageItemsRepeater;
		protected System.Web.UI.WebControls.PlaceHolder confirmation;
		protected System.Web.UI.WebControls.Repeater itemRepeater;
		protected System.Web.UI.WebControls.TextBox special;
		protected System.Web.UI.WebControls.Repeater methodsRepeater;
		#endregion

		private void Page_Load(object sender, System.EventArgs e) {
			storeContext = new StoreContext(this);

			// if user is not logged in, refer to login page
			if(!storeContext.UserLoggedIn){
				Response.Redirect("Login.aspx?Action=Login&Return=" + Request.Url);
			}

			if(!Page.IsPostBack){
				// ...
				addressRepeater.DataBind();
			}
		}

		/// <summary>
		/// Get the context object for this page.
		/// </summary>
		public StoreContext StoreContext{
			get {
				return storeContext;
			}
		}

		/// <summary>
		/// Handles a button click when user chooses to use an existing address
		/// </summary>
		protected void UseAddress(object sender, EventArgs e){
			Button button = (Button)sender;
			TextBox id = (TextBox)button.Parent.FindControl("id");
			if(billingAddress.Visible){
				ViewState.Add("billingContact",Int32.Parse(id.Text));
				billingAddress.Visible = false;
				shippingAddress.Visible = true;
			} else {
				ViewState.Add("shippingContact",Int32.Parse(id.Text));
				shippingAddress.Visible = false;
				RequestCard();
			}
		}
		
		/// <summary>
		/// Handles button click when user chooses to enter a new address
		/// </summary>
		protected void NewAddress(object sender, EventArgs e){
			Page.Validate();
			if(Page.IsValid){

				//...

				if(billingAddress.Visible){
					ViewState.Add("billingContact",newContactInfo);
					billingAddress.Visible = false;
					shippingAddress.Visible = true;

					name.Text = "";
					street.Text = "";
					city.Text = "";
					state.SelectedIndex = 0;
					zip.Text = "";
					zip4.Text = "";
					phone.Text = "";
					email.Text = "";

					// ...
					addressRepeater.DataBind();
				} else {
					ViewState.Add("shippingContact",newContactInfo);
					shippingAddress.Visible = false;
					RequestCard();
				}
			}
		}

		/// <summary>
		/// Called when form is ready to collect credit card information
		/// </summary>
		private	void RequestCard(){
			address.Visible = false;
			creditCard.Visible = true;
			
			// ...
			
			creditCardRepeater.DataBind();
		}

		/// <summary>
		/// Handles a button click when a user decides to use an existing credit card
		/// </summary>
		protected void UseCard(object sender, EventArgs e){
			Button button = (Button)sender;
			TextBox id = (TextBox)button.Parent.FindControl("id");
			ViewState.Add("card",Int32.Parse(id.Text));
			creditCard.Visible = false;
			shippingOptions.Visible = true;

			BuildPackages();
		}

		/// <summary>
		/// Handles a button click when a user enters new credit card information
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void NewCard(object sender, EventArgs e){

			// ...

			ViewState.Add("card",newCard);
			creditCard.Visible = false;
			shippingOptions.Visible = true;
			BuildPackages();
		}

		/// <summary>
		/// Breaks an order down into packages.
		/// </summary>
		/// <remarks>
		/// Factors an order into packages with priority given to warehouses that can ship
		/// larger portions of the order then to warehouses that are closer to the destination address.
		/// </remarks>
		private void BuildPackages(){

			// ... good luck!

			AssignMethodToPackage();

		}

		/// <summary>
		/// Assigns a shipping method to each pacakge
		/// </summary>
		/// <remarks>
		/// May be called multiple times if there are more than one package related to an order.
		/// </remarks>
		private void AssignMethodToPackage(){

			// ... good luck! 

			DataBind();
		}

		/// <summary>
		/// Handles a button click when a user assigns a method to a package
		/// </summary>
		protected void ChooseOption(object sender, EventArgs e){
			Button senderButton = (Button)sender;
			RepeaterItem item = (RepeaterItem)senderButton.Parent;

			TextBox idBox = (TextBox)item.FindControl("id");
			TextBox priceBox = (TextBox)item.FindControl("price");

			// ...

			if(true /* done choosing methods for packages */){
				// then we are done with packages, lets move onto funner stuff

				shippingOptions.Visible = false;
				confirmation.Visible = true;

				// ...

				DataBind();
			} else {
				// there are more packages that need methods assigned to them
				AssignMethodToPackage();
			}
		}		

		/// <summary>
		/// Handles the final click of the confirm order button
		/// </summary>
		/// <remarks>
		/// Performs the main transation including inserting the invoice, packages, items, whatnot into the database.
		/// </remarks>
		protected void ConfirmOrder(object sender, EventArgs e){

			int newInvoice = 0;

			// ...

			Response.Redirect("Order.aspx?Invoice=" + newInvoice);

		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
